"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports["default"] = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));

var _react = require("react");

var _Code = _interopRequireDefault(require("@rsuite/icons/Code"));

var _classnames = _interopRequireDefault(require("classnames"));

var _MarkdownRenderer = _interopRequireDefault(require("./MarkdownRenderer"));

var _CodeEditor = _interopRequireDefault(require("./CodeEditor"));

var _parseHTML = _interopRequireDefault(require("./utils/parseHTML"));

var _Preview = _interopRequireDefault(require("./Preview"));

var _canUseDOM = _interopRequireDefault(require("./utils/canUseDOM"));

var _excluded = ["children", "dependencies", "editor", "theme", "editable", "transformOptions", "sourceCode", "renderToolbar", "onChange", "beforeCompile", "compiler", "afterCompile"],
    _excluded2 = ["classPrefix", "icon", "className", "buttonClassName"];

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var React = require('react');

var ReactDOM = require('react-dom');

var defaultTransformOptions = {
  jsc: {
    parser: {
      syntax: 'ecmascript',
      jsx: true
    }
  }
};
var CodeView = React.forwardRef(function (props, ref) {
  var children = props.children,
      dependencies = props.dependencies,
      _props$editor = props.editor,
      editor = _props$editor === void 0 ? {} : _props$editor,
      _props$theme = props.theme,
      theme = _props$theme === void 0 ? 'light' : _props$theme,
      _props$editable = props.editable,
      isEditable = _props$editable === void 0 ? false : _props$editable,
      _props$transformOptio = props.transformOptions,
      transformOptions = _props$transformOptio === void 0 ? defaultTransformOptions : _props$transformOptio,
      sourceCode = props.sourceCode,
      renderToolbar = props.renderToolbar,
      onChange = props.onChange,
      beforeCompile = props.beforeCompile,
      compiler = props.compiler,
      afterCompile = props.afterCompile,
      rest = (0, _objectWithoutPropertiesLoose2["default"])(props, _excluded);
  var classPrefix = editor.classPrefix,
      codeIcon = editor.icon,
      editorClassName = editor.className,
      buttonClassName = editor.buttonClassName,
      editorProps = (0, _objectWithoutPropertiesLoose2["default"])(editor, _excluded2);

  var _useState = (0, _react.useState)(false),
      initialized = _useState[0],
      setInitialized = _useState[1];

  var transfrom = (0, _react.useRef)(null);
  (0, _react.useEffect)(function () {
    if (!_canUseDOM["default"]) {
      return;
    }

    Promise.resolve().then(function () {
      return _interopRequireWildcard(require('@swc/wasm-web'));
    }).then( /*#__PURE__*/function () {
      var _ref = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(module) {
        return _regenerator["default"].wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return module["default"]();

              case 2:
                transfrom.current = module.transformSync;
                setInitialized(true);

              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }());
  }, []);
  var sourceStr = children !== null && children !== void 0 && children.__esModule ? children["default"] : sourceCode;

  var _ref2 = (0, _parseHTML["default"])(sourceStr) || {},
      code = _ref2.code,
      beforeHTML = _ref2.beforeHTML,
      afterHTML = _ref2.afterHTML;

  var _useState2 = (0, _react.useState)(isEditable),
      editable = _useState2[0],
      setEditable = _useState2[1];

  var _useState3 = (0, _react.useState)(false),
      hasError = _useState3[0],
      setHasError = _useState3[1];

  var _useState4 = (0, _react.useState)(null),
      errorMessage = _useState4[0],
      setErrorMessage = _useState4[1];

  var _useState5 = (0, _react.useState)(null),
      compiledReactNode = _useState5[0],
      setCompiledReactNode = _useState5[1];

  (0, _react.useEffect)(function () {
    if (initialized) {
      executeCode(code);
    }
  }, [initialized, code]);
  var handleExpandEditor = (0, _react.useCallback)(function () {
    setEditable(!editable);
  }, [editable]);
  var handleError = (0, _react.useCallback)(function (error) {
    setHasError(true);
    setErrorMessage(error.message);
  }, []);

  var prefix = function prefix(name) {
    return classPrefix ? classPrefix + "-" + name : name;
  };

  var executeCode = (0, _react.useCallback)(function (pendCode) {
    if (pendCode === void 0) {
      pendCode = code;
    }

    if (!_canUseDOM["default"]) {
      return;
    }

    var originalRender = ReactDOM.render; // Redefine the render function, which will reset to the default value after `eval` is executed.
    // @ts-ignore

    ReactDOM.render = function (element) {
      setCompiledReactNode(element);
    };

    try {
      var statement = dependencies ? Object.keys(dependencies).map(function (key) {
        return "var " + key + "= dependencies." + key + ";";
      }) : [];
      var beforeCompileCode = (beforeCompile === null || beforeCompile === void 0 ? void 0 : beforeCompile(pendCode)) || pendCode;

      if (beforeCompileCode) {
        var _transfrom$current;

        var _ref3 = compiler ? compiler(beforeCompileCode) : (_transfrom$current = transfrom.current) === null || _transfrom$current === void 0 ? void 0 : _transfrom$current.call(transfrom, beforeCompileCode, transformOptions),
            compiledCode = _ref3.code;
        /* eslint-disable */


        eval(statement.join('\n') + " " + ((afterCompile === null || afterCompile === void 0 ? void 0 : afterCompile(compiledCode)) || compiledCode));
        /* eslint-enable */
      }
    } catch (err) {
      console.error(err);
    } finally {
      // Reset the render function to the original value.
      // @ts-ignore
      ReactDOM.render = originalRender;
    }
  }, [initialized, code, dependencies]);
  var handleCodeChange = (0, _react.useCallback)(function (code) {
    setHasError(false);
    setErrorMessage(null);
    onChange === null || onChange === void 0 ? void 0 : onChange(code);

    if (initialized) {
      executeCode(code);
    }
  }, [executeCode, initialized]);
  var codeButton = /*#__PURE__*/React.createElement("button", {
    role: "switch",
    "aria-checked": editable,
    "aria-label": "Show the full source",
    className: (0, _classnames["default"])(prefix('btn'), prefix('btn-xs'), buttonClassName),
    onClick: handleExpandEditor
  }, typeof codeIcon !== 'undefined' ? codeIcon : /*#__PURE__*/React.createElement(_Code["default"], {
    className: (0, _classnames["default"])(prefix('icon'), prefix('icon-code'))
  }));
  var showCodeEditor = editable && code && initialized;
  return /*#__PURE__*/React.createElement("div", (0, _extends2["default"])({
    ref: ref
  }, rest), /*#__PURE__*/React.createElement(_MarkdownRenderer["default"], null, beforeHTML), /*#__PURE__*/React.createElement("div", {
    className: "rcv-container"
  }, /*#__PURE__*/React.createElement(_Preview["default"], {
    hasError: hasError,
    errorMessage: errorMessage,
    onError: handleError
  }, compiledReactNode || /*#__PURE__*/React.createElement("div", null, "Loading...")), /*#__PURE__*/React.createElement("div", {
    className: "rcv-toolbar"
  }, renderToolbar ? renderToolbar(codeButton) : codeButton), showCodeEditor && /*#__PURE__*/React.createElement(_CodeEditor["default"], (0, _extends2["default"])({}, editorProps, {
    key: "jsx",
    onChange: handleCodeChange,
    className: (0, _classnames["default"])(editorClassName, 'rcv-editor'),
    editorConfig: {
      lineNumbers: true,
      theme: "base16-" + theme
    },
    code: code
  }))), /*#__PURE__*/React.createElement(_MarkdownRenderer["default"], null, afterHTML));
});
var _default = CodeView;
exports["default"] = _default;