/// <reference types="react" />
export interface CodeViewProps extends Omit<React.HTMLAttributes<HTMLElement>, 'onChange'> {
    /** Code editor theme, applied to CodeMirror */
    theme?: 'light' | 'dark';
    /** The code to be rendered is executed. Usually imported via markdown-loader. */
    children?: any;
    /** The code to be rendered is executed */
    sourceCode?: string;
    /** Dependent objects required by the executed code */
    dependencies?: object;
    /** Renders a code editor that can modify the source code */
    editable?: boolean;
    /** Editor properties */
    editor?: {
        className?: string;
        /** Add a prefix to the className of the buttons on the toolbar */
        classPrefix?: string;
        /** The className of the code button displayed on the toolbar */
        buttonClassName?: string;
        /** Customize the code icon on the toolbar */
        icon?: React.ReactNode;
    };
    /**
     * swc configuration
     * https://swc.rs/docs/configuration/compilation
     */
    transformOptions?: object;
    /** Customize the rendering toolbar */
    renderToolbar?: (buttons: React.ReactNode) => React.ReactNode;
    /** Callback triggered after code change */
    onChange?: (code?: string) => void;
    /**
     * A compiler that transforms the code. Use swc.transformSync by default
     * See https://swc.rs/docs/usage/wasm
     */
    compiler?: (code: string) => string;
    /** Executed before compiling the code */
    beforeCompile?: (code: string) => string;
    /** Executed after compiling the code */
    afterCompile?: (code: string) => string;
}
declare const CodeView: any;
export default CodeView;
