import _extends from "@babel/runtime/helpers/esm/extends";
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
var _excluded = ["children", "dependencies", "editor", "theme", "editable", "transformOptions", "sourceCode", "renderToolbar", "onChange", "beforeCompile", "compiler", "afterCompile"],
    _excluded2 = ["classPrefix", "icon", "className", "buttonClassName"];
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { useEffect, useState, useCallback, useRef } from 'react';
import CodeIcon from '@rsuite/icons/Code';
import classNames from 'classnames';
import MarkdownRenderer from './MarkdownRenderer';
import CodeEditor from './CodeEditor';
import parseHTML from './utils/parseHTML';
import Preview from './Preview';
import canUseDOM from './utils/canUseDOM';

var React = require('react');

var ReactDOM = require('react-dom');

var defaultTransformOptions = {
  jsc: {
    parser: {
      syntax: 'ecmascript',
      jsx: true
    }
  }
};
var CodeView = React.forwardRef(function (props, ref) {
  var children = props.children,
      dependencies = props.dependencies,
      _props$editor = props.editor,
      editor = _props$editor === void 0 ? {} : _props$editor,
      _props$theme = props.theme,
      theme = _props$theme === void 0 ? 'light' : _props$theme,
      _props$editable = props.editable,
      isEditable = _props$editable === void 0 ? false : _props$editable,
      _props$transformOptio = props.transformOptions,
      transformOptions = _props$transformOptio === void 0 ? defaultTransformOptions : _props$transformOptio,
      sourceCode = props.sourceCode,
      renderToolbar = props.renderToolbar,
      onChange = props.onChange,
      beforeCompile = props.beforeCompile,
      compiler = props.compiler,
      afterCompile = props.afterCompile,
      rest = _objectWithoutPropertiesLoose(props, _excluded);

  var classPrefix = editor.classPrefix,
      codeIcon = editor.icon,
      editorClassName = editor.className,
      buttonClassName = editor.buttonClassName,
      editorProps = _objectWithoutPropertiesLoose(editor, _excluded2);

  var _useState = useState(false),
      initialized = _useState[0],
      setInitialized = _useState[1];

  var transfrom = useRef(null);
  useEffect(function () {
    if (!canUseDOM) {
      return;
    }

    import('@swc/wasm-web').then( /*#__PURE__*/function () {
      var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(module) {
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return module["default"]();

              case 2:
                transfrom.current = module.transformSync;
                setInitialized(true);

              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }());
  }, []);
  var sourceStr = children !== null && children !== void 0 && children.__esModule ? children["default"] : sourceCode;

  var _ref2 = parseHTML(sourceStr) || {},
      code = _ref2.code,
      beforeHTML = _ref2.beforeHTML,
      afterHTML = _ref2.afterHTML;

  var _useState2 = useState(isEditable),
      editable = _useState2[0],
      setEditable = _useState2[1];

  var _useState3 = useState(false),
      hasError = _useState3[0],
      setHasError = _useState3[1];

  var _useState4 = useState(null),
      errorMessage = _useState4[0],
      setErrorMessage = _useState4[1];

  var _useState5 = useState(null),
      compiledReactNode = _useState5[0],
      setCompiledReactNode = _useState5[1];

  useEffect(function () {
    if (initialized) {
      executeCode(code);
    }
  }, [initialized, code]);
  var handleExpandEditor = useCallback(function () {
    setEditable(!editable);
  }, [editable]);
  var handleError = useCallback(function (error) {
    setHasError(true);
    setErrorMessage(error.message);
  }, []);

  var prefix = function prefix(name) {
    return classPrefix ? classPrefix + "-" + name : name;
  };

  var executeCode = useCallback(function (pendCode) {
    if (pendCode === void 0) {
      pendCode = code;
    }

    if (!canUseDOM) {
      return;
    }

    var originalRender = ReactDOM.render; // Redefine the render function, which will reset to the default value after `eval` is executed.
    // @ts-ignore

    ReactDOM.render = function (element) {
      setCompiledReactNode(element);
    };

    try {
      var statement = dependencies ? Object.keys(dependencies).map(function (key) {
        return "var " + key + "= dependencies." + key + ";";
      }) : [];
      var beforeCompileCode = (beforeCompile === null || beforeCompile === void 0 ? void 0 : beforeCompile(pendCode)) || pendCode;

      if (beforeCompileCode) {
        var _transfrom$current;

        var _ref3 = compiler ? compiler(beforeCompileCode) : (_transfrom$current = transfrom.current) === null || _transfrom$current === void 0 ? void 0 : _transfrom$current.call(transfrom, beforeCompileCode, transformOptions),
            compiledCode = _ref3.code;
        /* eslint-disable */


        eval(statement.join('\n') + " " + ((afterCompile === null || afterCompile === void 0 ? void 0 : afterCompile(compiledCode)) || compiledCode));
        /* eslint-enable */
      }
    } catch (err) {
      console.error(err);
    } finally {
      // Reset the render function to the original value.
      // @ts-ignore
      ReactDOM.render = originalRender;
    }
  }, [initialized, code, dependencies]);
  var handleCodeChange = useCallback(function (code) {
    setHasError(false);
    setErrorMessage(null);
    onChange === null || onChange === void 0 ? void 0 : onChange(code);

    if (initialized) {
      executeCode(code);
    }
  }, [executeCode, initialized]);
  var codeButton = /*#__PURE__*/React.createElement("button", {
    role: "switch",
    "aria-checked": editable,
    "aria-label": "Show the full source",
    className: classNames(prefix('btn'), prefix('btn-xs'), buttonClassName),
    onClick: handleExpandEditor
  }, typeof codeIcon !== 'undefined' ? codeIcon : /*#__PURE__*/React.createElement(CodeIcon, {
    className: classNames(prefix('icon'), prefix('icon-code'))
  }));
  var showCodeEditor = editable && code && initialized;
  return /*#__PURE__*/React.createElement("div", _extends({
    ref: ref
  }, rest), /*#__PURE__*/React.createElement(MarkdownRenderer, null, beforeHTML), /*#__PURE__*/React.createElement("div", {
    className: "rcv-container"
  }, /*#__PURE__*/React.createElement(Preview, {
    hasError: hasError,
    errorMessage: errorMessage,
    onError: handleError
  }, compiledReactNode || /*#__PURE__*/React.createElement("div", null, "Loading...")), /*#__PURE__*/React.createElement("div", {
    className: "rcv-toolbar"
  }, renderToolbar ? renderToolbar(codeButton) : codeButton), showCodeEditor && /*#__PURE__*/React.createElement(CodeEditor, _extends({}, editorProps, {
    key: "jsx",
    onChange: handleCodeChange,
    className: classNames(editorClassName, 'rcv-editor'),
    editorConfig: {
      lineNumbers: true,
      theme: "base16-" + theme
    },
    code: code
  }))), /*#__PURE__*/React.createElement(MarkdownRenderer, null, afterHTML));
});
export default CodeView;